"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostgresDialect = void 0;
const postgres_driver_js_1 = require("./postgres-driver.js");
const postgres_introspector_js_1 = require("./postgres-introspector.js");
const postgres_query_compiler_js_1 = require("./postgres-query-compiler.js");
const postgres_adapter_js_1 = require("./postgres-adapter.js");
const object_utils_js_1 = require("../../util/object-utils.js");
/**
 * PostgreSQL dialect that uses the [pg](https://node-postgres.com/) library.
 *
 * The {@link PostgresDialectConfig | configuration} passed to the constructor
 * is given as-is to the pg library's [Pool](https://node-postgres.com/api/pool)
 * constructor. See the following two links for more documentation:
 *
 * https://node-postgres.com/api/pool
 * https://node-postgres.com/api/client
 */
class PostgresDialect {
    #config;
    constructor(config) {
        this.#config = (0, object_utils_js_1.freeze)({ ...config });
    }
    createDriver() {
        return new postgres_driver_js_1.PostgresDriver(this.#config);
    }
    createQueryCompiler() {
        return new postgres_query_compiler_js_1.PostgresQueryCompiler();
    }
    createAdapter() {
        return new postgres_adapter_js_1.PostgresAdapter();
    }
    createIntrospector(db) {
        return new postgres_introspector_js_1.PostgresIntrospector(db);
    }
}
exports.PostgresDialect = PostgresDialect;
